<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="html" encoding="iso-8859-1" omit-xml-declaration="yes"/>

	<xsl:strip-space elements="status abstract chapter block list"/>

	<xsl:template match="/manual">
		<html>
			<head>
				<title><xsl:value-of select="@title"/></title>
				<meta name="description" content="{@title}"/>
			</head>
			<body>
				<h1><xsl:value-of select="@title"/></h1>
				<xsl:apply-templates select="status"/>
				<xsl:apply-templates select="abstract"/>
				<h2>Table of Content</h2>
				<xsl:call-template name="toc"/>
				<xsl:apply-templates select="chapter"/>
				<hr/>
				<p><xsl:value-of select="@revision"/></p>
			</body>
		</html>
	</xsl:template>

	<xsl:template match="status">
		<h2>Status of this Document</h2>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="abstract">
		<h2>Introduction</h2>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template name="toc">
		<xsl:if test="count(chapter)">
			<ol>
				<xsl:apply-templates select="chapter" mode="toc"/>
			</ol>
		</xsl:if>
	</xsl:template>

	<xsl:template match="chapter" mode="toc">
		<li><a href="#{@name}"><xsl:value-of select="@title"/></a></li>
		<xsl:if test="count(block)">
			<ul><xsl:apply-templates select="block" mode="toc"/></ul>
		</xsl:if>
	</xsl:template>

	<xsl:template match="chapter">
		<h3><a name="{@name}"><xsl:value-of select="@title"/></a></h3>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="block" mode="toc">
		<li>
			<a href="#{@name}"><xsl:value-of select="@title"/></a>
		</li>
	</xsl:template>

	<xsl:template match="block">
		<h4><a name="{@name}"><xsl:value-of select="@title"/></a></h4>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="para">
		<p><xsl:apply-templates/></p>
	</xsl:template>

	<xsl:template match="list">
		<xsl:if test="@type = 'numbered'">
			<ol>
				<xsl:apply-templates/>
			</ol>
		</xsl:if>
		<xsl:if test="not(@type = 'numbered')">
			<ul>
				<xsl:apply-templates/>
			</ul>
		</xsl:if>
	</xsl:template>

	<xsl:template match="item">
		<li><xsl:apply-templates/></li>
	</xsl:template>

	<xsl:template match="syntax">
		<code>
			<xsl:text>Syntax: </xsl:text>
			<xsl:value-of select="."/>
		</code>
	</xsl:template>

	<!-- ugly.. -->
	<xsl:template match="link">
		<a>
			<xsl:attribute name="href">
				<xsl:if test="@block">
					<xsl:text>#</xsl:text>
					<xsl:value-of select="@block"/>
				</xsl:if>
				<xsl:if test="@href">
					<xsl:value-of select="@href"/>
				</xsl:if>
			</xsl:attribute>
			<xsl:value-of select="."/>
		</a>
	</xsl:template>

</xsl:stylesheet>
