/* m_lusers.c -- count clients statistics;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

#ifndef lint
static char rcsid[] = "$Id: m_lusers.c,v 1.9 2002/06/16 11:13:46 hex Exp $";
#endif

#include "options.h"
#include "struct.h"
#include "send.h"
#include "numeric.h"
#include "h.h"
#include "channel.h" /* for IsULine() */
#include "common.h"

/* why the .... does the original variable get trashed?! */
static int max_global_users = 0;


/* parv[0] = sender, parv[1] = host/server mask, parv[2] = server to query */
int m_lusers(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int s_count, c_count, u_count, i_count;
	int o_count, m_client, m_server;
	char mydom_mask[HOSTLEN + 1];
	aClient *acptr;

	if (check_registered_user(sptr))
		return 0;

	if (parc > 2) {
		if (!BadPtr(server_mask) && !IsPrivileged(sptr)) {
			sendto_one(sptr, err_str(ERR_NOPRIVILEGES),
				me.name, parv[0]);
			return 0;
		}
		if (hunt_server(cptr, sptr, ":%s LUSERS %s :%s", 2, parc, parv)
			!= HUNTED_ISME) return 0;
	}

	strcpy(mydom_mask, "*");

	if (parc == 1) {
		s_count = lu_serv;
		c_count = lu_noninv;
		u_count = lu_unknown;
		i_count = lu_inv;
		o_count = lu_oper;
		m_client = lu_lu;
		m_server = lu_lserv;
		global_count = lu_cglobalu;
	}
	else {
		s_count = 0;
		c_count = 0;
		u_count = 0;
		i_count = 0;
		o_count = 0;
		m_client = 0;
		m_server = 0;
		collapse(parv[1]);
		for (acptr = client; acptr; acptr = acptr->next) {
			if (parc > 1)
				if (!IsServer(acptr) && acptr->user) {
					if (match(parv[1], acptr->srvptr->name))
						continue;
				}
				else
					if (match(parv[1], acptr->name))
						continue;

			switch (acptr->status) {
				case STAT_SERVER:
					if (MyConnect(acptr))
						++m_server;

				case STAT_ME:
					++s_count;
					break;

				case STAT_CLIENT:
					if (IsULine(acptr, acptr))
						break;
					if (IsOper(acptr))
						++o_count;
					if (MyConnect(acptr))
						++m_client;
					if (!IsInvisible(acptr))
						++c_count;
					else
						++i_count;
					break;
				default:
					++u_count;
					break;
			}
		}
	}

	sendto_one(sptr, rpl_str(RPL_LUSERCLIENT), me.name, parv[0],
		c_count, i_count, s_count);

	sendto_one(sptr, rpl_str(RPL_LUSEROP), me.name, parv[0],
		o_count);
	sendto_one(sptr, rpl_str(RPL_LUSERUNKNOWN), me.name, parv[0],
		u_count);
	sendto_one(sptr, rpl_str(RPL_LUSERCHANNELS), me.name, parv[0],
		lu_channel);
	sendto_one(sptr, rpl_str(RPL_LUSERME), me.name, parv[0],
		m_client, IsAnOper(sptr) ? m_server : 1);

	sendto_one(sptr, rpl_str(RPL_LOCALUSERS), me.name, parv[0],
		m_client, lu_mlu);
	sendto_one(sptr, rpl_str(RPL_GLOBALUSERS), me.name, parv[0],
		global_count, max_global_users /*lu_mglobalu*/);

	return 0;
}


void check_lusers(void) {
	aClient *acptr;
	lu_noninv = 0;
	lu_inv = 0;
	lu_serv = 0;
	lu_oper = 0;
	lu_unknown = 0;
	lu_channel = 0;
	lu_lu = 0;
	lu_lserv = 0;
	lu_clu = 0;
	lu_cglobalu = 0;

	for (acptr = client; acptr; acptr = acptr->next) {
		switch (acptr->status) {
			case STAT_SERVER:
				if (MyConnect(acptr))
					++lu_lserv;

			case STAT_ME:
				++lu_serv;
				break;

			case STAT_CLIENT:
				if (IsULine(acptr, acptr))
					break;
				if (IsOper(acptr))
					++lu_oper;
				if (MyConnect(acptr))
					++lu_lu;
				if (!IsInvisible(acptr))
					++lu_noninv;
				else
					++lu_inv;
				break;

			default:
				++lu_unknown;
				break;
		}
	}

	lu_clu = lu_lu;
	lu_cglobalu = lu_noninv + lu_inv;

	if (lu_clu > lu_mlu)
		lu_mlu = lu_lu;

	if (lu_cglobalu > lu_mglobalu)
		lu_mglobalu = lu_cglobalu;

	lu_channel = count_channels(&me);

	if (lu_mglobalu > max_global_users)
		max_global_users = lu_mglobalu;
	else if (lu_mglobalu < max_global_users) {
		sendto_realops("WARNING!  Trying to decrease MaxGlobalUsers "
			"counter from %d to %d - please report to #IRCD!",
			max_global_users, lu_mglobalu);
		lu_mglobalu = max_global_users;
	}
}
