/* m_map.c -- display network hierarchy;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

#ifndef lint
static char rcsid[] = "$Id: m_map.c,v 1.6 2002/06/14 18:07:36 hex Exp $";
#endif

#include "options.h"
#include "struct.h"
#include "send.h"
#include "numeric.h"
#include "h.h"
#include "common.h"

static void dump_map(aClient *cptr, aClient *server, char *mask,
	int prompt_length, int length)
{
	static char prompt[64];
	char *p = &prompt[prompt_length];
	int cnt = 0, local = 0, oper = 0;
	aClient *acptr;

	*p='\0';

	if (prompt_length > 60)
		sendto_one(cptr, rpl_str(RPL_MAPMORE), me.name,
			cptr->name, prompt, server->name);
	else {
		for (acptr = client; acptr; acptr = acptr->next) {
			if (IsPerson(acptr)) {
				++cnt;
				if (acptr->srvptr == server) {
					++local;
					if (IsAnOper(acptr))
						++oper;
				}
			}
		}
		sendto_one(cptr, rpl_str(RPL_MAP), me.name, cptr->name, prompt,
			length, server->name, local, oper, (local*100)/cnt );
		cnt = 0;
	}

	if (prompt_length > 0) {
		p[-1] = ' ';
		if (p[-2] == '`') p[-2] = ' ';
	}

	if (prompt_length > 60)
		return;

	strcpy(p, "|-");


	for (acptr = client; acptr ; acptr = acptr->next) {
		if (!IsServer (acptr) || strcmp(acptr->serv->up, server->name))
			continue;

		if (match(mask, acptr->name))
			acptr->flags &= ~FLAGS_MAP;
		else {
			acptr->flags |= FLAGS_MAP;
			++cnt;
		}
	}

	for (acptr = client; acptr ; acptr = acptr->next) {
		if (!(acptr->flags & FLAGS_MAP) || !IsServer (acptr)
			|| strcmp(acptr->serv->up, server->name))
			continue;

		if (--cnt == 0)
			*p = '`';

		dump_map(cptr, acptr, mask, prompt_length+2, length-2);
	}

	if (prompt_length > 0)
		p[-1] = '-';
}


/* m_map  -- by Run, partly rewitten by unilynx because we haven't got "dlinks",
 * whatever those may be :)  and to add some cute statistics to the standard
 * dump.  parv[0] = sender prefix, parv[1] = server mask
 */

int m_map(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	int longest = strlen(me.name);

	if (check_registered_user(sptr))
		return 0;

	if (!BadPtr(server_mask) && !OPIsOper(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (parc < 2)
		parv[1] = "*";

	for (acptr = client; acptr; acptr = acptr->next)
		if (IsServer(acptr) && ( strlen(acptr->name) + acptr->hopcount*2 ) > longest)
			longest = strlen(acptr->name) + acptr->hopcount*2;

	if (longest > 60)
		longest = 60;

	longest += 2;
	dump_map(sptr, &me, parv[1], 0, longest);
	sendto_one(sptr, rpl_str(RPL_MAPEND), me.name, parv[0]);

	return 0;
}
