/* m_oper.c -- get IRC Operator priviledes;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

#ifndef lint
static char rcsid[] = "$Id: m_oper.c,v 1.10 2002/07/14 12:19:20 hex Exp $";
#endif

#include <stdlib.h>
#include <stdio.h>
#ifndef _WIN32
# include <syslog.h>
#endif
#include "struct.h"
#include "send.h"
#include "numeric.h"
#include "h.h"
#include "md5.h"
#include "common.h" /* for opers_log */

static void putlog(const char *uid, int ok, aClient *acptr)
{
	FILE *log;

	if (!opers_log)
		return;

#ifndef _WIN32
	if (!strcmp(opers_log, "syslog")) {
		syslog(LOG_INFO, "%s %s (%s) by %s[%s@%s] (pid=%d)",
			myctime(time(NULL)), ok ? "OPER" : "FAILED OPER",
			uid, acptr->name, acptr->user->username,
			acptr->sockhost, getpid());
		return;
	}
#endif

	if (log = fopen(opers_log, "a")) {
		fprintf(log, "%s %s (%s) by %s[%s@%s]\n",
			myctime(time(NULL)), ok ? "OPER" : "FAILED OPER",
			uid, acptr->name, acptr->user->username,
			acptr->sockhost);
		fclose(log);
	}
}


/* parv[0] = sender prefix, parv[1] = oper name, parv[2] = oper password */
int m_oper(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int flog;
	aConfItem *aconf;
	char tmpbuf[1024];

	if (check_registered_user(sptr))
		return 0;

	if (MyConnect(sptr) && IsAnOper(sptr)) {
		sendto_one(sptr, ":%s 381 :You are an IRC Operator already!",
			me.name, parv[0]);
		return 0;
	}

	if (!IsServer(cptr) && (BadPtr(parv[1]) || BadPtr(parv[2]) || parc < 3)) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "OPER");
		return 0;
	}

	/* if message arrived from server, trust it, and set to oper */
	if ((IsServer(cptr) || IsMe(cptr)) && !IsOper(sptr)) {
		sptr->umodes |= UMODE_OPER;
		sendto_serv_butone(cptr, ":%s MODE %s :+o", parv[0], parv[0]);
		if (IsMe(cptr))
			sendto_one(sptr, rpl_str(RPL_YOUREOPER), me.name, parv[0]);
		return 0;
	}

	if (!(aconf = find_conf_exact(parv[1], sptr->username, (*sptr->vhost) ? sptr->vhost : sptr->sockhost, CONF_OPS)) && !(aconf = find_conf_exact(parv[1], sptr->username, inetntoa((char *)&cptr->ip), CONF_OPS))) {
		sendto_one(sptr, err_str(ERR_NOOPERHOST), me.name, parv[0]);
		sendto_realops("Failed OPER attempt by %s[%s@%s]", parv[0], sptr->user->username, (*sptr->vhost) ? sptr->vhost : sptr->sockhost);
		sptr->since += 7;
		return 0;
	}

	if ((aconf->status & CONF_OPS) && !strcasecmp(MDPassword(parv[2]), *aconf->passwd == '+' ? aconf->passwd+1 : aconf->passwd) && !attach_conf(sptr, aconf)) {
		int old = (sptr->umodes & ALL_UMODES);
		char *s;

		s = strchr(aconf->host, '@');
		*s++ = '\0';
		if (!(aconf->port & OFLAG_OPER))
			SetLocOp(sptr);
		else
			SetOper(sptr);
		sptr->oflag = aconf->port;
		*--s = '@';
		sendto_ops("%s (%s!%s@%s) is now operator (%c)", parv[1],
			parv[0], sptr->user->username, sptr->user->host,
			IsOper(sptr) ? 'o' : 'O');
		sptr->umodes |= (UMODE_SERVNOTICE|UMODE_WALLOP|UMODE_FAILOP|UMODE_FLOOD);
		if (sptr->oflag & OFLAG_ADMIN)
			sptr->umodes |= UMODE_ADMIN;
		if (sptr->oflag & OFLAG_SADMIN)
			sptr->umodes |= UMODE_SADMIN;
		send_umode_out(cptr, sptr, old);
		sendto_one(sptr, rpl_str(RPL_YOUREOPER), me.name, parv[0]);

#if defined(USE_SYSLOG) && defined(SYSLOG_OPER)
		syslog(LOG_INFO, "OPER (%s) by (%s!%s@%s)", name, parv[0],
			sptr->user->username, (*sptr->vhost) ?
			sptr->vhost : sptr->sockhost);
#endif

		if (IsPerson(sptr))
			putlog(parv[1], 1, sptr);
	}
	else {
		detach_conf(sptr, aconf);
		sendto_one(sptr,err_str(ERR_PASSWDMISMATCH),me.name, parv[0]);

		sendto_realops("Failed OPER attempt by %s[%s@%s] using UID %s",
			parv[0], sptr->user->username,
			(*sptr->vhost) ? sptr->vhost : sptr->sockhost, parv[1]);

		sendto_serv_butone(&me, ":%s GLOBOPS :Failed OPER attempt by "
			"%s[%s@%s] using UID %s", me.name, parv[0],
			sptr->user->username,
			(*sptr->vhost) ? sptr->vhost : sptr->sockhost, parv[1]);

		sptr->since += 7;

		if (IsPerson(sptr))
			putlog(parv[1], 0, sptr);
	}

	hmask_update(sptr);

	return 0;
}
