/* m_part.c -- remove a user from a channel;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

#ifndef lint
static char rcsid[] = "$Id: m_part.c,v 1.7 2002/06/14 08:21:44 hex Exp $";
#endif

#include <string.h>
#include "struct.h"
#include "h.h"
#include "numeric.h"
#include "send.h"
#include "channel.h"

const char *PartFmt = ":%s PART %s", *PartFmt2 = ":%s PART %s :%s";

/* parv[0] = sender prefix, parv[1] = channel,
   parv[2] = comment (added by Lefler) */
int m_part(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	Link *lp;
	char *p = NULL, *name, partmsg[BUFSIZE];
	char *comment = (parc > 2 && parv[2]) ? parv[2] : NULL;

	if (check_registered_user(sptr))
		return 0;

	sptr->flags &= ~FLAGS_TS8;

	if (parc < 2 || parv[1][0] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS),
			me.name, parv[0], "PART");
		return 0;
	}

	for (; (name = strtoken(&p, parv[1], ",")); parv[1] = NULL) {
		if (!(chptr = get_channel(sptr, name, 0))) {
			sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL),
				me.name, parv[0], name);
			continue;
		}

		if (check_channelmask(sptr, cptr, name))
			continue;

		/* Do not use IsMember here: zombies must be able to part too */
		if (!(lp = find_user_link(chptr->members, sptr))) {
			/* Normal to get get when our client did a kick for a
			   remote client (who sends back a PART), so check for
			   remote client or not --Run */
			if (MyClient(sptr))
				sendto_one(sptr, err_str(ERR_NOTONCHANNEL),
					me.name, parv[0], name);

			continue;
		}

		/* strip colors from +c channels */
		if (comment && ColorBlindChannel(chptr)) {
			strncpyzt(partmsg, comment, sizeof(partmsg));
			strip_color(partmsg);
			comment = partmsg;

			if (!*comment)
				comment = NULL;
		}

		/* strip comments if banned or moderated */
		if (comment && can_send(cptr, chptr))
			comment = NULL;

		/* Remove user from the old channel (if any) */
		if (!comment)
			sendto_match_servs(chptr, cptr,
				(char*)PartFmt, parv[0], name);
		else
			sendto_match_servs(chptr, cptr,
				(char*)PartFmt2, parv[0], name, comment);

		if (!(lp->flags & CHFL_ZOMBIE)) {
			if (!comment)
				sendto_channel_butserv(chptr, sptr,
					(char*)PartFmt, parv[0], name);
			else
				sendto_channel_butserv(chptr, sptr,
					(char*)PartFmt2, parv[0], name, comment);
		}
		else if (MyClient(sptr)) {
			if (!comment)
				sendto_one(sptr, (char*)PartFmt, parv[0], name);
			else
				sendto_one(sptr, (char*)PartFmt2, parv[0],
					name, comment);
		}

		remove_user_from_channel(sptr, chptr);
	}

	return 0;
}
