/* m_ping.c -- link aliveness check, s2s lag measurement;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

#ifndef lint
static char rcsid[] = "$Id: m_ping.c,v 1.11 2002/07/11 16:46:52 hex Exp $";
#endif

/* notes on P?NG forwarding, depending on the number of parameters:

   :sender PING target source -- travelling to the source
   :sender PING target source timestamp -- travelling to the target
   :target PONG sender source timestamp -- travelling to the source
   :target PONG sender source timestamp ballast -- on the was to the sender.
 */

#include "struct.h"
#include "send.h"
#include "numeric.h"
#include "h.h"

aClient * find_match_server(char *mask)
{
	aClient *acptr;

	if (BadPtr(mask))
		return NULL;

	for (acptr = client, collapse(mask); acptr; acptr = acptr->next) {
		if (!IsServer(acptr) && !IsMe(acptr))
			continue;

		if (!match(mask, acptr->name))
			break;
	}

	return acptr;
}


int m_ping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient dummy; /* for routing changes */
	char timebuf[16];

	if (check_registered(sptr))
		return 0;

	if (parc < 2 || BadPtr(parv[1])) {
		sendto_one(sptr, err_str(ERR_NOORIGIN), me.name, parv[0]);
		return 0;
	}

	/* first process local PINGs */
	if (MyClient(sptr)) {
		aClient *tcptr, *scptr;

		/* non-opers are not allowed to do that */
		if (!IsOper(sptr))
			goto bounce;

		/* find target, if it's not a valid server - bounce */
		if (!(tcptr = find_match_server(parv[1])))
			goto bounce;
		else {
			/* unexpand the target name */
			parv[1] = tcptr->name;
		}

		/* find the source, if it's not valid - assume myself */
		if (!(scptr = parc > 2 ? find_match_server(parv[2]) : NULL)) {
			scptr = &me;
			parv[2] = scptr->name;
		}

		/* now we always have a fixed number of parameters */
		parc = 3;

		/* if we're the target - swap the parameters */
		if (IsMe(tcptr)) {
			char *tmp;

			/* if we're the source as well - bounce */
			if (IsMe(scptr))
				goto bounce;

			tmp = parv[2];
			parv[2] = parv[1];
			parv[1] = tmp;
		}
	}

	/* if there's no timestamp - it's travelling to the source */
	if (parc == 3) {
		/* it wasn't me - forwarding further */
		if (hunt_server(cptr, sptr, ":%s PING %s %s", 2, parc, parv)
			!= HUNTED_ISME) return 0;

		/* it was me, append the timestamp */
		sprintf(timebuf, "%u", get_ticks());
		parv[parc++] = timebuf;

		/* now, since the direction might have changed, reset the
		   sources, otherwise the command might be discarded by
		   further hunt_server() */
		memset((void*)&dummy, 0, sizeof(dummy));
		cptr = sptr = &dummy;
	}

	/* if there's a time stamp - it's travelling to the final destination */
	if (parc == 4) {
		char *pparv[4];

		if (hunt_server(cptr, sptr, ":%s PING %s %s %s", 1, parc, parv)
			!= HUNTED_ISME) return 0;

		/* so I am the destination.  instead of implementing the whole
		   PONG thing here, we construct something that looks like a
		   PONG from a remote server, and call the handler. */

		pparv[0] = parv[1]; /* original target */
		pparv[1] = parv[0]; /* original sender */
		pparv[2] = parv[2]; /* original source */
		pparv[3] = parv[3]; /* original timestamp */

#ifdef DEBUGMODE
		sendto_syschan("debug", "got a ping, replying: %s %s %s %s",
			pparv[0], pparv[1], pparv[2], pparv[3]);
#endif

		return m_pong(&me, &me, parc, pparv);
	}

bounce:
	sendto_one(sptr, ":%s PONG %s :%s", me.name, parv[0],
		BadPtr(parv[1]) ? "" : parv[1]);

	return 0;
}
