/* m_protoctl.c -- protocol setup during the connection phase;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

#ifndef lint
static char rcsid[] = "$Id: m_protoctl.c,v 1.12 2002/06/14 21:41:31 hex Exp $";
#endif

#include "options.h"
#include "struct.h"
#include "common.h"
#include "h.h"
#include "zlink.h"

/* parv[0] = Sender prefix, parv[1+] = Options */
int m_protoctl(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int i;
	char proto[128], *options, *equal;
	struct ConfItem *aconf;

	/* But we already GOT a protoctl msg! */
	if (GotProtoctl(sptr))
		return 0;

	cptr->flags |= FLAGS_PROTOCTL;

	for (i = 1; i < parc; ++i) {
		strncpy(proto, parv[i], sizeof(proto));
		proto[sizeof(proto)-1] = '\0';
		equal = (char *) strchr(proto, '=');
		if (equal == NULL)
			options = "";
		else {
			options = equal + 1;
			equal[0] = '\0';
		}

		if (!strcasecmp(proto, "NOQUIT")) {
			SetNoQuit(cptr);
		}
		else if (!strcasecmp(proto, "TOKEN")) {
			SetToken(cptr);
		}
		else if (!strcasecmp(proto, "LINKCTL")) {
			char *s;

			for (s = strtok(options, ","); s; s = strtok(NULL, ",")) {
				if (!strcasecmp(s, "ZIP"))
					SetZipable(cptr);
			}
		}
		else if (!BadPtr(network_name) && !strcasecmp(proto, "NETWORK")) {
			if (strcasecmp(options, network_name)) {
				sendto_one(cptr, "ERROR :We are %s, not %s!  "
					"Go away to your network!",
					network_name, options);
				sendto_realops("Network mismatch with %s (our: "
					"%s, their: %s), closing connection.",
					cptr->name, network_name, options);
				return exit_client(cptr, cptr, cptr,
					"Network name mismatch");
			}
		}
	}

	/* an older version links in - PROTOCTL before introducing itself */
	if (!IsServer(sptr))
		return 0;

	/* server link is officially established now, adjust the protocol */

	if ((aconf = find_conf(cptr->confs, cptr->name, CONF_NOCONNECT_SERVER))
	&& IsZipable(cptr) && (aconf->port & PROTO_ZIP)) {
		sendto_one(cptr, "LINKCTL +ZIP");
		SetZipOut(cptr);
		if (!(cptr->serv->zip_out = zip_create_output_session())) {
			sendto_one(cptr, "ERROR :Zipped link "
				"initialization failed.");
			return 0;
		}
	}

	return m_server_burst(sptr);
}
