/* m_squit.c -- disconnect a server from the network;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

#ifndef lint
static char rcsid[] = "$Id: m_squit.c,v 1.4 2002/06/14 08:33:40 hex Exp $";
#endif

#include "struct.h"
#include "h.h"
#include "numeric.h"
#include "msg.h"

/* parv[0] = sender prefix, parv[1] = server name, parv[parc-1] = comment */
int m_squit(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aConfItem *aconf;
	aClient *acptr;
	char *server, *comment = (parc > 2 && !BadPtr(parv[parc-1]))
		? parv[parc-1] : cptr->name;

	if (check_registered(sptr))
		return 0;

	if (!IsPrivileged(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (parc > 1) {
		server = parv[1];

		/* To accomodate host masking, a squit for a masked server
		   name is expanded if the incoming mask is the same as
		   the server name for that link to the name of link. */
		while ((*server == '*') && IsServer(cptr)) {
			aconf = cptr->serv->nline;
			if (!aconf)
				break;
			if (!mycmp(server, my_name_for_link(me.name, aconf)))
				server = cptr->name;
			break; /* WARNING is normal here */
		}

		/* The following allows wild cards in SQUIT. Only usefull
		   when the command is issued by an oper. */
		for (acptr = client; (acptr = next_client(acptr, server)); ) {
			if (IsServer(acptr) || IsMe(acptr))
				break;
			acptr = acptr->next;
		}
		if (acptr && IsMe(acptr)) {
			acptr = cptr;
			server = cptr->sockhost;
		}
	}

	/* don't squit clients */
	else if (MyClient(sptr)) {
		send_syntax(sptr, MSG_SQUIT,
			"SQUIT server_mask [:comment ...]");
		return 0;
	}
	else {
		/* This is actually protocol error. But, well, closing
		   the link is very proper answer to that... */
		server = cptr->sockhost;
		acptr = cptr;
	}

	/* SQUIT semantics is tricky, be careful...
	 *
	 * The old (irc2.2PL1 and earlier) code just cleans away the
	 * server client from the links (because it is never true
	 * "cptr == acptr".
	 *
	 * This logic here works the same way until "SQUIT host" hits
	 * the server having the target "host" as local link. Then it
	 * will do a real cleanup spewing SQUIT's and QUIT's to all
	 * directions, also to the link from which the orinal SQUIT
	 * came, generating one unnecessary "SQUIT host" back to that
	 * link.
	 *
	 * One may think that this could be implemented like
	 * "hunt_server" (e.g. just pass on "SQUIT" without doing
	 * nothing until the server having the link as local is
	 * reached). Unfortunately this wouldn't work in the real life,
	 * because either target may be unreachable or may not comply
	 * with the request. In either case it would leave target in
	 * links--no command to clear it away. So, it's better just
	 * clean out while going forward, just to be sure.
	 *
	 * ...of course, even better cleanout would be to QUIT/SQUIT
	 * dependant users/servers already on the way out, but
	 * currently there is not enough information about remote
	 * clients to do this...   --msa
	 */

	if (!acptr) {
		sendto_one(sptr, err_str(ERR_NOSUCHSERVER),
			me.name, parv[0], server);
		return 0;
	}

	if (MyClient(sptr) && ((!OPIsOper(sptr) && !MyConnect(acptr))
	|| (!OPIsAnOper(sptr) && MyConnect(acptr)))) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	/*  Notify all opers, if my local link is remotely squitted */
	if (MyConnect(acptr) && !IsAnOper(cptr)) {
		sendto_locfailops("Received SQUIT %s from %s (%s)",
			acptr->name, get_client_name(sptr,FALSE), comment);

		sendto_serv_butone(&me, ":%s WALLOPS :Received SQUIT %s "
			"from %s (%s)", me.name, server,
			get_client_name(sptr,FALSE), comment);

#if defined(USE_SYSLOG) && defined(SYSLOG_SQUIT)
		syslog(LOG_DEBUG, "SQUIT From %s : %s (%s)", parv[0],
			server, comment);
#endif
	}
	else if (MyConnect(acptr)) {
		sendto_locfailops("Received SQUIT %s from %s (%s)",
			acptr->name, get_client_name(sptr,FALSE), comment);

		sendto_serv_butone(&me, ":%s WALLOPS :Received SQUIT %s "
			"from %s (%s)", me.name, acptr->name,
			get_client_name(sptr,FALSE), comment);
	}
	if (IsAnOper(sptr)) {
		/* It was manually /squit'ed by a human being(we hope),
		 * there is a very good chance they don't want us to
		 * reconnect right away.  -Cabal95 */
		acptr->flags |= FLAGS_SQUIT;
	}

	return exit_client(cptr, acptr, sptr, comment);
}
