/* m_userhost.c -- get users' addresses;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

#ifndef lint
static char rcsid[] = "$Id: m_userhost.c,v 1.5 2002/06/14 08:33:40 hex Exp $";
#endif

#include "struct.h"
#include "send.h"
#include "numeric.h"
#include "h.h"

/* m_userhost added by Darren Reed 13/8/91 to aid clients and reduce
 * the need for complicated requests like WHOIS. It returns user/host
 * information only (no spurious AWAY labels or channels). */
int m_userhost(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int catsize, resid;
	char *p = NULL, *s, *curpos, buf[1024];
	aClient *acptr;

	if (check_registered(sptr))
		return 0;

	if (parc > 2)
		m_userhost(cptr, sptr, parc-1, parv+1);

	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS),
			me.name, parv[0], "USERHOST");
		return 0;
	}

	/* use curpos to keep track of where we are in the output buffer,
	 * and use resid to keep track of the remaining space in the
	 * buffer */

	curpos = buf;
	curpos += sprintf(curpos, rpl_str(RPL_USERHOST), me.name, parv[0]);
	resid = sizeof(buf) - (curpos - buf) - 1; /* remaining space */

	/* for each user found, print an entry if it fits. */
	for (s = strtoken(&p, parv[1], " "); s; ) {
		if ((acptr = find_person(s, NULL))) {
			catsize = strlen(acptr->name)
				+ (IsAnOper(acptr) ? 1 : 0)
				+ 3 + strlen(acptr->user->username)
				+ strlen(acptr->user->host) + 1;

			if (catsize <= resid) {
				char *h;

				if (MyConnect(acptr) && acptr == sptr)
					h = acptr->sockhost;
				else
					h = IsHMask(acptr) ? acptr->user->vhost
					: acptr->user->host;

				curpos += sprintf(curpos, "%s%s=%c%s@%s ",
					acptr->name,
					IsAnOper(acptr) ? "*" : "",
					IsAway(acptr) ? '-' : '+',
					acptr->user->username, h);
				resid -= catsize;
			}

			/* notify opers they're userhosted */
			if (SendServNotice(acptr) && acptr != sptr)
				sendto_one(acptr, ":%s NOTICE %s :USERHOST "
					"from %s!%s@%s", me.name, acptr->name,
					sptr->name, sptr->user->username,
					sptr->user->host);
		}

		s = strtoken(&p, (char *)NULL, " ");
	}

	/* because of some trickery here, we might have the string end in
	 * "...:" or "foo " (note the trailing space)
	 * If we have a trailing space, nuke it here. */
	if (*--curpos != ':')
		*curpos = '\0';
	sendto_one(sptr, "%s", buf);
	return 0;
}
