/* support.c -- misc;
 * Copyright (C) 2001-2002, ForestNet IRC Network
 * This is a free software; see file `license' for details.
 */

#ifndef lint
static char lint[] = "$Id: support.c,v 1.7 2002/06/22 17:46:26 hex Exp $";
#endif

#include "config.h"
#ifdef DYNIXPTX
#include <sys/timers.h>
#include <stddef.h>
#endif
#include "struct.h"
#include "common.h"
#include "sys.h"
#ifdef _WIN32
#include <io.h>
#else

extern  int errno; /* ...seems that errno.h doesn't define this everywhere */
#endif
#ifndef CLIENT_COMPILE
extern  void    outofmemory();
#endif

#ifdef NEED_STRTOKEN
/*
**      strtoken.c --   walk through a string of tokens, using a set
**                      of separators
**                      argv 9/90
**
**      $Id: support.c,v 1.7 2002/06/22 17:46:26 hex Exp $
*/

char *strtoken(char **save, char *str, char *fs)
{
	char *pos = *save;  /* keep last position across calls */
	char *tmp;

	if (str)
		pos = str; /* new string scan */

	while (pos && *pos && strchr(fs, *pos) != NULL)
		pos++; /* skip leading separators */

	if (!pos || !*pos)
		return (pos = *save = NULL); /* string contains only sep's */

	tmp = pos; /* now, keep position of the token */

	while (*pos && strchr(fs, *pos) == NULL)
		pos++; /* skip content of the token */

	if (*pos)
		*pos++ = '\0'; /* remove first sep after the token */
	else
		pos = NULL; /* end of string */

	*save = pos;
	return (tmp);
}
#endif /* NEED_STRTOKEN */

#ifdef  NEED_STRTOK
/* NOT encouraged to use! */

char *strtok(char *str, char *fs)
{
	static char *pos;

	return strtoken(&pos, str, fs);
}

#endif /* NEED_STRTOK */

#if defined(NEED_STRERROR) && !defined(_WIN32)
/*
**      strerror - return an appropriate system error string to a given errno
**
**                 argv 11/90
**      $Id: support.c,v 1.7 2002/06/22 17:46:26 hex Exp $
*/

char *strerror(int err_no)
{
	extern char *sys_errlist[];  /* Sigh... hopefully on all systems */
	extern int sys_nerr;

	static char buff[40];
	char *errp;

	errp = (err_no > sys_nerr ? (char *)NULL : sys_errlist[err_no]);

	if (errp == (char *)NULL) {
		errp = buff;
#ifndef _WIN32
		sprintf(errp, "Unknown Error %d", err_no);
#else
		switch (err_no) {
			case WSAECONNRESET:
				sprintf(errp, "Connection reset by peer");
				break;
			default:
				sprintf(errp, "Unknown Error %d", err_no);
				break;
		}
#endif
	}
	return errp;
}

#endif /* NEED_STRERROR */

/*
**      inetntoa  --    changed name to remove collision possibility and
**                      so behaviour is gaurunteed to take a pointer arg.
**                      -avalon 23/11/92
**      inet_ntoa --    returned the dotted notation of a given
**                      internet number (some ULTRIX don't have this)
**                      argv 11/90).
**      inet_ntoa --    its broken on some Ultrix/Dynix too. -avalon
**      $Id: support.c,v 1.7 2002/06/22 17:46:26 hex Exp $
*/

char *inetntoa(char *in)
{
	static char buf[16];
	u_char *s = (u_char *)in;
	int a,b,c,d;

	a = (int)*s++;
	b = (int)*s++;
	c = (int)*s++;
	d = (int)*s++;
	(void) sprintf(buf, "%d.%d.%d.%d", a,b,c,d );

	return buf;
}

#ifdef NEED_INET_NETOF
/*
**      inet_netof --   return the net portion of an internet number
**                      argv 11/90
**      $Id: support.c,v 1.7 2002/06/22 17:46:26 hex Exp $
**
*/

int inet_netof(in)
struct in_addr in;
{
    int addr = in.s_net;

    if (addr & 0x80 == 0)
        return ((int) in.s_net);

    if (addr & 0x40 == 0)
        return ((int) in.s_net * 256 + in.s_host);

    return ((int) in.s_net * 256 + in.s_host * 256 + in.s_lh);
}
#endif /* NEED_INET_NETOF */


#if defined(DEBUGMODE) && !defined(CLIENT_COMPILE)
void dumpcore(msg, p1, p2, p3, p4, p5, p6, p7, p8, p9)
char *msg, *p1, *p2, *p3, *p4, *p5, *p6, *p7, *p8, *p9;
{
        static  time_t  lastd = 0;
        static  int     dumps = 0;
        char    corename[12];
        time_t  now;
        int     p;

        now = time(NULL);

        if (!lastd)
                lastd = now;
        else if (now - lastd < 60 && dumps > 2)
                (void)s_die();
        if (now - lastd > 60)
            {
                lastd = now;
                dumps = 1;
            }
        else
                dumps++;
# ifndef _WIN32
        p = getpid();
        if (fork()>0) {
                kill(p, 3);
                kill(p, 9);
        }
        write_pidfile();
        (void)sprintf(corename, "core.%d", p);
        (void)rename("core", corename);
        Debug((DEBUG_FATAL, "Dumped core : core.%d", p));
        sendto_ops("Dumped core : core.%d", p);
# endif
        Debug((DEBUG_FATAL, msg, p1, p2, p3, p4, p5, p6, p7, p8, p9));
        sendto_ops(msg, p1, p2, p3, p4, p5, p6, p7, p8, p9);
                (void)s_die();
}
#endif

static  char    *marray[20000];
static  int     mindex = 0;

#define SZ_EX   (sizeof(char *) + sizeof(size_t) + 4)
#define SZ_CHST (sizeof(char *) + sizeof(size_t))
#define SZ_CH   (sizeof(char *))
#define SZ_ST   (sizeof(size_t))


/*
** read a string terminated by \r or \n in from a fd
**
** Created: Sat Dec 12 06:29:58 EST 1992 by avalon
** Returns:
**      0 - EOF
**      -1 - error on read
**     >0 - number of bytes returned (<=num)
** After opening a fd, it is necessary to init dgets() by calling it as
**      dgets(x,y,0);
** to mark the buffer as being empty.
*/
int     dgets(fd, buf, num)
int     fd, num;
char    *buf;
{
        static  char    dgbuf[8192];
        static  char    *head = dgbuf, *tail = dgbuf;
        register char   *s, *t;
        register int    n, nr;

        /*
        ** Sanity checks.
        */
        if (head == tail)
                *head = '\0';
        if (!num)
            {
                head = tail = dgbuf;
                *head = '\0';
                return 0;
            }
        if (num > sizeof(dgbuf) - 1)
                num = sizeof(dgbuf) - 1;
dgetsagain:
        if (head > dgbuf)
            {
                for (nr = tail - head, s = head, t = dgbuf; nr > 0; nr--)
                        *t++ = *s++;
                tail = t;
                head = dgbuf;
            }
        /*
        ** check input buffer for EOL and if present return string.
        */
        if (head < tail &&
            ((s = strchr(head, '\n')) || (s = strchr(head, '\r'))) && s < tail)
            {
                n = MIN(s - head + 1, num);     /* at least 1 byte */
dgetsreturnbuf:
                bcopy(head, buf, n);
                head += n;
                if (head == tail)
                        head = tail = dgbuf;
                return n;
            }

        if (tail - head >= num)         /* dgets buf is big enough */
            {
                n = num;
                goto dgetsreturnbuf;
            }

        n = sizeof(dgbuf) - (tail - dgbuf) - 1;
        nr = read(fd, tail, n);
        if (nr == -1)
            {
                head = tail = dgbuf;
                return -1;
            }
        if (!nr)
            {
                if (head < tail)
                    {
                        n = MIN(tail - head, num);
                        goto dgetsreturnbuf;
                    }
                head = tail = dgbuf;
                return 0;
            }
        tail += nr;
        *tail = '\0';
        for (t = head; (s = strchr(t, '\n')); )
            {
                if ((s > head) && (s > dgbuf))
                    {
                        t = s-1;
                        for (nr = 0; *t == '\\'; nr++)
                                t--;
                        if (nr & 1)
                            {
                                t = s+1;
                                s--;
                                nr = tail - t;
                                while (nr--)
                                        *s++ = *t++;
                                tail -= 2;
                                *tail = '\0';
                            }
                        else
                                s++;
                    }
                else
                        s++;
                t = s;
            }
        *tail = '\0';
        goto dgetsagain;
}

#ifdef NEED_STRLCPY
size_t strlcpy(char *dst, const char *src, size_t size)
{
	strncpy(dst, src, size);
	dst[size - 1] = '\0';
	return strlen(dst);
}
#endif

#ifdef NEED_STRLCAT
size_t strlcat(char *dst, const char *src, size_t size)
{
	strncat(dst, src, size);
	dst[size - 1] = '\0';
	return strlen(dst);
}
#endif
