/******************************************************************************/
/* Internet Relay Chat daemon, see readme for further information             */
/* (c) 2001-2002 forestnet.org                                                */
/******************************************************************************/

#ifndef lint
static char rcsid[] = "$Id: whowas.c,v 1.10 2002/06/14 18:07:36 hex Exp $";
#endif

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "whowas.h"
#include "h.h"

static aName was[NICKNAMEHISTORYLENGTH];
static int ww_index = 0;

void add_history(aClient *cptr)
{
	Link *lp;
	aName ntmp, *np = &ntmp, *np2;

	strncpyzt(np->ww_nick, cptr->name, NICKLEN+1);
	strncpyzt(np->ww_info, cptr->info, REALLEN+1);
	strncpyzt(np->ww_server, cptr->srvptr->name, HOSTLEN+1);

	np->ww_umodes=cptr->umodes;
	np->ww_user = cptr->user;
	np->ww_logout = time(NULL);
	np->ww_online = (cptr->from != NULL) ? cptr : NULL;
	np->ww_user->refcnt++;

	np2 = &was[ww_index];
	if (np2->ww_user)
		free_user(np2->ww_user, np2->ww_online);

	/*
	 * New whowas handling, we keep a list of what whowas entries
	 * are "used" by a client, in its cptr structure.  This means
	 * that when we overwrite a whowas entry, we have to remove the
	 * relative pointer in the client. -Cabal95
	 */
	if (np2->ww_online) {
		Link *last = NULL;

		for (lp = np2->ww_online->history; lp;
			 last = lp, lp = lp->next)
				if (lp->value.whowas == np2)
					break;

		/* Sanity check, never trust anything */
		if (lp) {
			if (last)
				last->next = lp->next;
			else
				np2->ww_online->history = lp->next;

			free_link(lp);
		}
	}

	bcopy((char *)&ntmp, (char *)np2, sizeof(aName));

	/* Add this whowas entry into the clients history list */
	lp = make_link();
	lp->value.whowas = np2;
	lp->next = cptr->history;
	cptr->history = lp;

	ww_index++;
	if (ww_index >= NICKNAMEHISTORYLENGTH)
			ww_index = 0;
	return;
}

/*
** get_history
**      Return the current client that was using the given
**      nickname within the timelimit. Returns NULL, if no
**      one found...
*/
aClient *get_history(char *nick, time_t timelimit)
{
	int i = 0;
	aName *wp, *wp2;

	if (ww_index == 0)
		wp = wp2 = &was[NICKNAMEHISTORYLENGTH - 1];
	else
		wp = wp2 = &was[ww_index - 1];
	timelimit = time(NULL)-timelimit;

	do {
		if (!mycmp(nick, wp->ww_nick) && wp->ww_logout >= timelimit)
			break;
		if (wp == was) {
			i = 1;
			wp = &was[NICKNAMEHISTORYLENGTH - 1];
		}
		else
			wp--;
	} while (wp != wp2);

	if (wp != wp2 || !i)
		return (wp->ww_online);

	return NULL;
}

void off_history(aClient *cptr)
{
	Link *lp, *next;

	for (lp = cptr->history; lp; lp = next) {
		next = lp->next;
		lp->value.whowas->ww_online = NULL;
		free_link(lp);
	}

	cptr->history = NULL;

	return;
}

void initwhowas()
{
	int i;

	for (i = 0; i < NICKNAMEHISTORYLENGTH; i++)
		bzero((char *)&was[i], sizeof(aName));

	return;
}


/* parv[0] = sender prefix, parv[1] = nickname queried */
int m_whowas(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aName *wp, *wp2 = NULL;
	anUser *up = NULL;
	int j = 0, max = -1;
	char *p, *nick, *s;

	if (check_registered_user(sptr))
		return 0;

	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NONICKNAMEGIVEN), me.name, parv[0]);
		return 0;
	}

	if (!BadPtr(server_mask) && MyClient(sptr) && parc > 2
	&& !IsPrivileged(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}

	if (parc > 2)
		max = atoi(parv[2]);

	if (parc > 3 && hunt_server(cptr,sptr,":%s WHOWAS %s %s :%s", 3,parc,parv))
		return 0;

	for (s = parv[1]; (nick = strtoken(&p, s, ",")); s = NULL) {
		wp = wp2 = &was[ww_index - 1];

		do {
			if (wp < was)
				wp = &was[NICKNAMEHISTORYLENGTH - 1];
			if (mycmp(nick, wp->ww_nick) == 0) {
				up = wp->ww_user;
				sendto_one(sptr, rpl_str(RPL_WHOWASUSER), me.name, parv[0], wp->ww_nick, up->username, ((wp->ww_umodes & UMODE_HMASK) && !IsAnOper(sptr)) ? up->vhost : up->host, wp->ww_info);
				sendto_one(sptr, rpl_str(RPL_WHOISSERVER), me.name, parv[0], wp->ww_nick,
					(BadPtr(server_mask) || IsAnOper(sptr)) ? wp->ww_server : server_mask,
					myctime(wp->ww_logout));
				j++;
			}
			if (max > 0 && j >= max)
				break;
			wp--;
		} while (wp != wp2);

		if (up == NULL)
			sendto_one(sptr, err_str(ERR_WASNOSUCHNICK), me.name, parv[0], nick);
		up=NULL;

		if (p)
			p[-1] = ',';
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFWHOWAS), me.name, parv[0], parv[1]);
	return 0;
}


void count_whowas_memory(int *wwu, int *wwa, u_long *wwam)
{
	anUser *tmp;
	int i, j, u = 0, a = 0;
	u_long  am = 0;

	for (i = 0; i < NICKNAMEHISTORYLENGTH; i++)
		if ((tmp = was[i].ww_user))
			if (!was[i].ww_online) {
				for (j = 0; j < i; j++)
					if (was[j].ww_user == tmp)
						break;
				if (j < i)
					continue;
				u++;
			}
	*wwu = u;
	*wwa = a;
	*wwam = am;

	return;
}



